//
//  jbunpf.c
//
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"

extern FILE *dbg;
extern int wnb, wc;
extern char cj[][JSentLen], cz[][CzLen], md[][JSentLen], mz[][ZokuLen],
	sj[], oj[], vr[], sz[], oz[],vz[], Dbg;
extern char en[][JSentLen];
char wd[WordLen];
static char op = 0x28, cp = 0x29;
char *plu = "addendum,addenda;alga,algae;alto,altos;alumna,alumnae;alumus,alumni;analysis,analyses;antelope,antelopes;antenna,antennas;apex,apexes;appendix,appendixes;aquarium,aquariums;archipelago,archipelagos;automation,automatons;axis,axes;bacillus,bacilli;bacterium,bacteria;banjo,banjoes;basis,bases;brther-in-law,brothers-in-law;buffalo,buffalos;bureau,bureaus;cactus,cacti;calf,calves;cargo,cargos;cello,cellos;chamois,chamois;chassis,chassis;child,children;codex,codices;command,commandos;concerto,concertos;corpus,corpora;crisis,crises;craftsman,craftsmen;criterion,criteria;man,men;shelf,shelves;woman,women;";

static char bungo[14][18] = {"ł","ł( 2)","(ł 3)",
	"Ȃ( 3)","͂Ȃ","Ȃ","͂Ȃ( 3)",
	"͖( 3)","Ȃ( 3)","( 3)",
	"","( 2)","낤","ɂȂ( 3)"},
 	kougo[14][20] = {"ł","ł","ł","͂܂ł",
	"͂܂","܂","͂܂ł","͗L܂ł",
	"܂ł","L܂ł","܂","܂",
	"ł傤","ɂ܂ł"},
 	mzn[9][3] = {"","","","","","","","",""},
 	rny[9][3] = {"","","","","","","","",""},
	shs[9][3] = {"","","","","","","","",""},
        mei[9][3] = {"","","","","","","","",""},
        kanyu[9][5] = {"","","","Ƃ","̂","","낤",
	"","ڂ"},
	onb[9][5] = {"","","","","","","",
	"",""},
	kak[9][5] = {"","","","","","","",
	"",""},
	ksm[32][5] = {"","˂","","","","o","",
	"ς","Q","","f","","","","","",
	"","","Ă","ɂ","˂","݂","߂","","",
	"","","","","ł","т","ׂ"};

past(yk, sw)
char yk[];
int  sw;
{
    int  k, j, ln;
    char *pt;
    if(strncmp(yk+1, "also ", 5) == 0) return;
    if(strcmp(yk+1, "there be") == 0){ strcpy(yk+7, "was"); return; }
    if(strcmp(yk+1, "there is") == 0){ strcpy(yk+7, "was"); return; }
    if(strcmp(yk+1, "there are") == 0){ strcpy(yk+7, "were"); return; }
    if(strncmp(yk+1, "is ", 3) == 0){ ptreplace(yk+1, 2, "was"); return; }
    if(strncmp(yk+1, "are ", 4) == 0){ ptreplace(yk+1, 3, "were"); return; }
    if(strncmp(yk+1, "has ", 4) == 0 || strcmp(yk+1, "has") == 0){ ptreplace(yk+1, 3, "have"); }
    if(strncmp(yk+1, "play ", 5) == 0){ ptreplace(yk+1, 4, "played"); }
    if((pt = (char*)strchr(yk, ' '))) ;
    else pt = (char*)strchr(yk, '\0');
    ptcpy(wd, yk+1, pt);
    ln = strlen(wd);
    if(irregVerb(sw) == 1){ ptreplace(yk+1, ln, wd); return; }
//    if(strncmp(wd, yk+1, ln) != 0){ ptreplace(yk+1, ln, wd); return; }
    if(strcmp(wd, "could") == 0 || strcmp(wd, "should") == 0) return;
    if(strcmp(wd, "has") == 0) return;
    if(*(pt-1) == 'e') ptinsert(pt, "d");    
    else if(*(pt-1) == 'y') ptreplace(pt-1, 1, "ied");    
    else if(*(pt-1) == 'p') ptinsert(pt, "ped");    
    else if(*(pt-1) == 'n' && *(pt-2) != 'g' ) ptinsert(pt, "ned");    
    //    else if(*(pt-1) == 'n' && strcmp(wd, "learn") && strcmp(wd, "mention") && strcmp(wd, "join") ) ptinsert(pt, "ned");    
    else ptinsert(pt, "ed");
}
/*
genkei(yk)
char yk[];
{
    int  ln, d;
    char *gobi, pars[JWordLen], *qt;
    *pars = '\0';
    ln = strlen(yk);
    gobi = yk+ln-2;
    if(strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0){
      if(strcmp(yk+ln-6, "傫") == 0) { strcpy(gobi, "");    goto RTN; }
      strcpy(gobi, "ł");	goto RTN;
    }
    else{
      if(strcmp(gobi, "") == 0) goto RTN;
    }
    if(yk[ln-1] != ')'){
      if(qt = (char*)strstr(yk, " (")){
	if(*(qt-1) != ')') goto RTN;
	strcpy(pars, qt);	*qt = '\0';
      }
    }
    if(qt = (char*)index(yk, '(')){
      ln = strlen(yk);	d = yk[ln-2]-'0';
      *(yk+ln-3) = '\0';	ptptcut(qt-(d*2), qt);
    }
    RTN:	if(*pars) strcat(yk, pars);
}

paren(yk)
char yk[];
{
    int  kk, ln, j = 0;
    char Er, *dums, *pt1, *pt2;
    ln = strlen(yk);
    if(yk[ln-1] == cp){
      for(kk = ln-2; kk >= 0; kk--)
	if(yk[kk] == op){ j = kk+1;	return(j); }
      Er = '\0';	puts("( ȂB@G[");
      while(! Er){
	puts("̃L[ĉB");	Er = getchar();
      }
    }
    dums = yk+ln-2;
    if(strcmp(dums, "j") == 0){
      pt1 = yk;
      while(pt2 = (char*)strstr(pt1, "i")) pt1 = pt2+2;
      if(pt1 == yk){
	puts("i@ȂBG[");	Er = '\0';
	while(! Er){
	  puts("̃L[ĉB");
	  Er = getchar();
	}
      }
      else {
	j = 1;	pt1 -= 2;
	while(yk != pt1--) ++j;
      }
      return(j);
    }
    return(j);
}
*/
akitsume(){
    int  i, j, ct;
    for(i = 0; i <= wnb; i++)
      if(cj[i][0] == '\0') goto TUME;
    return;
    TUME:	ct = i-1;
    if(i == wnb){ wnb--;	return; }
    for(j = i+1; j <= wnb; j++)
      if(cj[j][0] != '\0'){ ct++;
	strcpy(cj[ct], cj[j]);
	strcpy(cz[ct], cz[j]);	strcpy(en[ct], en[j]);
      }
    wnb = ct;
    cj[wnb+1][0] = '\0';
// printf("--- akitsume --- en[i-1] = %s en[i] = %s \n", en[i-1], en[i]);
}

transf(pt)
char *pt;
{
    int  i, lnb, j;
    char cmd[60], kaz[60], *pcm, buf[JSentLen];
    i = *(pt-1)-'0';
    if(pcm = (char *)index(pt+2, ';')) ptcpy(cmd, pt+1, pcm);
    if(strcmp(cmd, "-i") == 0){
//      kakudel(md[i]);	return;
    }
    lnb = strlen(cmd)-1;
    if(cmd[lnb] == '+'){   // #xx+; ̏ꍇ
      cmd[lnb] = '\0';
      if(strcmp(cmd, "SJ$") == 0){
        ptinsert(md[i]+1, " ");  ptinsert(md[i]+1, sj+1);  return;
      }
      if(strcmp(cmd, "VR$") == 0){
        ptinsert(md[i]+1, " ");  ptinsert(md[i]+1, vr );  return;
      }
      strcat(cmd, " ");  ptinsert(md[i]+1, cmd);  return;
    }
    if(strcmp(cmd, "&T") == 0){
  if(Dbg == 'Y') fprintf(dbg, "--- &T --- i = %d md = %s mz = %s \n", i, md[i], mz[i]);
      if(strstr(md[i]+1, "wh") && !strstr(md[i]+1, "whi") && strncmp(md[i]+1, "wh", 2) != 0) return;
      if(strncmp(md[i]+1, "whether ", 8) == 0) return;
      if(strncmp(md[i]+1, "many ", 5) == 0) return;
      if(strncmp(md[i]+1, "only ", 5) == 0 || strncmp(md[i]+1, "nation", 6) == 0) goto JMP1; // the
      if(strchr(mz[i], 'y') ) goto JMP2;
      if(strchr(mz[i], 'K') && strchr(mz[i], 'B')) return;
      if(strchr(mz[i], 'K') || strchr(mz[i], 'n') || strchr(mz[i], 'x') || strchr(mz[i], 't') || strstr(mz[i], "X") || strstr(mz[i], "") )
        return; 
    JMP2:
      // if(isupper(md[i][1])) return; -> 'American'̏ꍇȂǂ
      if(strncmp(md[i]+1, "the ", 4) == 0 || strncmp(md[i]+1, "my ", 3) == 0
      || strncmp(md[i]+1, "Mr. ", 4) == 0 || strncmp(md[i]+1, "Mrs. ", 5) == 0
      || strncmp(md[i]+1, "our ", 4) == 0 || strncmp(md[i]+1, "such ", 5) == 0
      || strncmp(md[i]+1, "you ", 4) == 0 || strncmp(md[i]+1, "it ", 3) == 0
      || strncmp(md[i]+1, "your ", 5) == 0 || strncmp(md[i]+1, "her ", 4) == 0
      || strncmp(md[i]+1, "his ", 4) == 0 || strncmp(md[i]+1, "one's ", 6) == 0
      || strncmp(md[i]+1, "to ", 3) == 0 || strncmp(md[i]+1, "someone", 7) == 0
      || strncmp(md[i]+1, "how ", 4) == 0 || strncmp(md[i]+1, "what ", 5) == 0
      || strncmp(md[i]+1, "where ", 6) == 0 || strncmp(md[i]+1, "( ", 2) == 0
      || strncmp(md[i]+1, "an ", 3) == 0 || strncmp(md[i]+1, "a ", 2) == 0
      || strncmp(md[i]+1, "some ", 5) == 0 || strncmp(md[i]+1, "any ", 4) == 0
      || strncmp(md[i]+1, "more ", 5) == 0
      || strncmp(md[i]+1, "every ", 6) == 0 || strncmp(md[i]+1, "1 ", 2) == 0
      || strncmp(md[i]+1, "not ", 4) == 0 || strncmp(md[i]+1, "last ", 5) == 0
      || strncmp(md[i]+1, "all ", 4) == 0 || strncmp(md[i]+1, "both ", 5) == 0
      || !strncmp(md[i]+1, "this ", 5) || strncmp(md[i]+1, "that ", 5) == 0
      || strncmp(md[i]+1, "anything ", 9) == 0
      || strncmp(md[i]+1, "something", 9) == 0
      || strncmp(md[i]+1, "school", 6) == 0
      || strcmp(md[i]+1, "chess") == 0 || strcmp(md[i]+1, "music") == 0
      || strcmp(md[i+1]+1, "be made of") == 0)
        return;
      if(strstr(md[i]+1, "day") && !strchr(md[i]+1, ' ')) return;
      if(strstr(md[i]+1, "ing") && !strchr(md[i]+1, ' ')) return;
      if(strstr(md[i]+1, "ing ")) return;
    JMP:
      // Ȃ
      if(strchr(mz[i], 'z')){
        strcpy(buf, "some ");
        strcat(buf, md[i]+1);
        strcpy(md[i]+1, buf);
        return;
      }
      // V́@ꏊ@^⎌
      if(strchr(mz[i], 'B') || strchr(mz[i], 'w')
      || strstr(md[i]+1, "wh")
//      || strncmp(md[i]+1, "that ", 5) == 0
      || strncmp(md[i]+1, "an ", 3)
      || strncmp(md[i]+1, "a ", 2) || strncmp(md[i]+1, "on ", 3) ){
        if(strncmp(md[i]+1, "to ", 3) == 0) return;
	if(strchr(mz[i], 'y')) goto AT;
      JMP1:
        strcpy(buf, "the ");
        strcat(buf, md[i]+1);
        strcpy(md[i]+1, buf);
        return;
      }
    AT:
      if(strstr(mz[i], "") || strchr(mz[i], 'n') ) return;
      if(strstr(mz[i], "") == 0 || strstr(mz[i], "P") || strchr(mz[i], 'y')){
        if(md[i+1][0] == '7') strcpy(buf, "the ");
        else if(strstr(mz[i], ""))  return;
        else if(strchr("aiueoAIUEO", md[i][1]) || strncmp(md[i]+1, "hour", 4) == 0) strcpy(buf, "an ");
        else strcpy(buf, "a ");
        strcat(buf, md[i]+1);
        strcpy(md[i]+1, buf);
        return;
      }
      else ninsyo(md[i], mz[i], md[i], mz[i]);
    }
    if(strncmp(cmd, "&J-1", 4) == 0){
      j = -1;
      ninsyo(md[i], mz[i], md[j], mz[j]);    return;
    }
    if(strncmp(cmd, "&J", 2) == 0){
      j = *(pt+3) - '0';
      ninsyo(md[i], mz[i], md[j], mz[j]);    return;
    }
    if(strncmp(cmd, "&", 1) == 0){
      ninsyo(md[i], mz[i], cmd, mz[i]);    return;
    }
    if(strncmp(cmd, "ONE&", 4) == 0){
      pcm = (char*)strstr(md[i]+1, "one");
      if(pcm != 0){
        oneTan(cmd+4, pcm);    return;
      }
    }
    if(strcmp(cmd, "SINKO") == 0){ sinko(md[i]);	return; }
//    if(strcmp(cmd, "RENYO") == 0){ renyo(i);	return; }
    if(strcmp(cmd, "HIKAKU") == 0){ hikaku(md[i]);	return; }
    if(strcmp(cmd, "SAIJYO") == 0){ saijyo(md[i]);	return; }
    if(strcmp(cmd, "PAST") == 0){ past(md[i], 1);	return; }
    if(strcmp(cmd, "PPLE") == 0){ past(md[i], 2);	return; }
//    if(strcmp(cmd, "GENKE") == 0){ genkei(md[i]);	return; }
    if(strcmp(cmd, "NEGA") == 0){ nega(md[i]);	return; }
//    if(strcmp(cmd, "GIMU") == 0){ gimu(md[i]);	return; }
//    if(strcmp(cmd, "RARER") == 0){
//      genkei(md[i]);	rareru(md[i]);	return;
//    }
//    if(strcmp(cmd, "SASER") == 0){ saser(md[i]);	return; }
//    if(strcmp(cmd, "DES") == 0 && strcmp(cj[wc+i+1], "9̂ŁA")){
//      desu(md[i]);	return;
//    }
//    if(strcmp(cmd, "DEKIR") == 0){ dekiru(md[i]);       return; }
//    if(strcmp(cmd, "KANYU") == 0){ kanyuugobi(md[i]);	return; }
    if(strcmp(cmd, "SJ") == 0){ sopresv(sj, sz, i);	return; }
    if(strcmp(cmd, "OJ") == 0){ sopresv(oj, oz, i);	return; }
    if(strcmp(cmd, "VR") == 0){ vrpresv(i);	return; }
    if(strcmp(cmd, "+SJ$") == 0){ add(*(pt-2), i, sj);    return; }
    if(strcmp(cmd, "+OJ$") == 0){ add(*(pt-2), i, oj);    return; }
    if(strncmp(cmd, "+J", 2) == 0 && isdigit(*(pt+3))){
      j = *(pt+3) - '0';
      add(*(pt-2), i, md[j]);    return;
    }
    if(cmd[0] == '-'){ minus(cmd+1, *(pt-2), i);	return; }
    if(cmd[0] == '+'){ add(*(pt-2), i, cmd);	return; }
    if(strcmp(cmd, "Z") == 0){
      if(*(pt-2) == 'J'){ strcpy(md[i], "Z");	return; }
      if(*(pt-2) == 'T'){ strcpy(cz[wc+i], "");	return; }
    }
    if(strcmp(cmd, "KAZ1") == 0){
      kazu3(i, kaz);	return;
    }
    if(strcmp(cmd, "KAZ2") == 0){ kazu2(i, kaz);	return; }
//    if(strcmp(cmd, "K&K") == 0){ sei_mei(md[i], md[i+1]);	return; }
}

sopresv(bj, bz, i)
int  i;
char bj[], bz[];
{
    int  t, j, ln, lz, l3;
    char ctck[60], cz3[60], bjq[JSentLen], *ptj, c;
 if(Dbg == 'Y') fprintf(dbg, "--- soprev --- cz = %s \n", cz[wc+i]);
    strcpy(cz3, cz[wc+i]);
    if(*bz == 'b'){
      ptj = (char*)index(bz+1, 'b');
      strcpy(ctck, ptj);	strcat(ctck, "b");
    }
    else{
      if(*bj == '1'){
	*ctck = 'b';	*(ctck+1) = '\0';
	strcat(ctck, bz);	strcat(ctck, "b");
      }
      else *ctck = '\0';
    }
    lz = strlen(ctck)-2;
    l3 = strlen(cz3)-1;
    if(lz+l3 > 27){
      j = 0;	*bz = *ctck;
      for(t = 1; t <= lz; t++){
	if(! index(bz, ctck[t])) bz[++j] = ctck[t];
      }
      bz[++j] = 'b';	j = 0;	*ctck = *cz3;
      for(t = 1; t <= 29; t++) ctck[t] = '\0';
      for(t = 0; t <= l3; t++){
	if(! index(ctck, cz3[t])) ctck[++j] = cz3[t];
      }
      strcpy(cz[wc+i], ctck);
      if(strlen(bz) + strlen(ctck) > 30){
	printf("\n The length of bz excedes 30 !! \n");
	printf(" p܂ [Y/N]= ");	c = '\0';
	while(c != 'y' && c != 'n') c = tolower(getchar());
        if(c != 'y') exit(1);
	return;
      }
    }
    else strcpy(bz, ctck);
    strcat(bz, cz[wc+i]);
    strcpy(bjq, md[i]);
    ln = strlen(bjq);
    if(ptj = (char*)strstr(bjq, "i")){
      ptptcut(bjq+1, ptj+1);	ln = strlen(bjq);
      bjq[ln-2] = '\0';	ln -= 2;	*bjq = '1';
    }
    if(ln > 13 && (ptj = (char*)strstr(bjq+3, "fƂ"))) *(ptj+2) = '\0';
    ptj = (char*)strstr(bjq, "");
    while(ptj && *(ptj+2) && (ptj != (char*)strstr(ptj, "̂"))
    && (ptj != (char*)strstr(ptj, "̕"))){
      ptptcut(bjq+1, ptj+1);	ptj = (char*)strstr(bjq, "");
    }
    ln = strlen(bjq);
    if((ptj = (char*)strstr(bjq, "j")) && strlen(ptj) >= 4)
      ptptcut(bjq+1, ptj+1);
    if(ln < JWordLen) strcpy(bj, bjq);
    else ptcpy(bj, bjq, bjq+WordLen-2);
}

vrpresv(i)
int  i;
{
    int  ln;
    char dums[100], mw[3], *pt;
    if(strlen(md[i]) > 100) return;;
    if(*vr == '\0'){
      ln = strlen(md[i]);
      if(ln > JWordLen-2){
	if(pt = (char*)strstr(md[i], "i")) ptcpy(vr, md[i]+1, pt);
	else ptcpy(vr, md[i]+1, md[i]+WordLen-1);
      }
      else strcpy(vr, md[i]+1);
      strcpy(vz, cz[wc+i]);
    }
    else{
      ln = strlen(vr);
      left(dums, vr, ln-2);
      if(strcmp(dums, "")==0) strcpy(dums, vr);
      if(((! strstr(md[i], dums)) && strlen(md[i]) < 45) ||
      (strcmp(dums, "ł")==0 && (! index(cz[wc+i], 'v')))){
	if(pt = (char*)strstr(md[i], "A")) strcpy(vr, pt+2);
	else strcpy(vr, md[i]+1);
	strcpy(vz, cz[wc+i]);
      }
    }
    if(pt = (char*)index(vr, '>')) ptptcut(vr, pt);
    left(mw, vr, 2);
    if(strcmp(mw, "")==0 || strcmp(mw, "")==0 || strcmp(mw, "")==0 ||
    strcmp(mw, "")==0 || strcmp(mw, "")==0) ptcut(vr, 2);
    if(pt = (char*)strstr(vr, "̓")) ptcut(pt, 6);
    ln = strlen(vr);
//    if(vr[ln-1] == ')') genkei(vr);
}

ninsyo(mdi, mzi, mdj, mzj)
char mdi[], mzi[], mdj[], mzj[];
{
    char *pt, mdjb[JSentLen];
    int i;

 if(Dbg == 'Y') fprintf(dbg, "--- ninsyo --- mdi = %s mdj = %s \n", mdi,mdj);
    for(i = strlen(mdi); i< JSentLen; i++)  mdi[i] = '\0';
    if(*mdi != '3' && *mdi != '5' && *mdi != '6' && strncmp(mdi+1, "be ", 3)) goto NINS0;
    // ωꂪ 
    strcpy(mdjb, mdj+1);
    if(strncmp(mdi+1, "can ", 4) == 0) return;
    if(strncmp(mdi+1, "cannot ", 7) == 0) return;
    if(strncmp(mdi+1, "can't ", 6) == 0) return;
    if(strncmp(mdi+1, "may ", 4) == 0) return;
    if(strncmp(mdi+1, "will ", 5) == 0) return;
    if(strncmp(mdi+1, "must ", 5) == 0) return;
    if(strncmp(mdi+1, "also ", 5) == 0) return;
    if(strncmp(mdi+1, "should ", 7) == 0) return;
    if(strncmp(mdi+1, "would ", 6) == 0) return;
    if(strncmp(mdi+1, "almost ", 7) == 0) return;
    if(strncmp(mdi+1, "there are", 9) == 0) return;
    if(strncmp(mdi+1, "there be", 8) == 0 ){
      if(strstr(mzj, "") ){
	ptreplace(mdi+7, 2, "are");    return;
      }
	ptreplace(mdi+7, 2, "is");    return;
    }
    if(strncmp(mdi+1, "be ", 3) == 0 || strcmp(mdi+1, "be") == 0){
      if(strncmp(mdj+1, "either", 6) == 0 && (pt =(char*)strrchr(mdj+1, ' '))){
        strcpy(mdjb, pt+1);
      }
      if(strcmp(mdjb, "I") == 0){
        ptreplace(mdi+1, 3, "am ");    return;
      }
      else if(strcmp(mdjb, "it") == 0 || strcmp(mdjb, "he") == 0){
        ptreplace(mdi+1, 3, "is ");    return;
      }
      else if(strstr(mzj, "") || strstr(mzj, "F") || strcmp(mdjb, "you") == 0 || strcmp(mdjb, "we") == 0 || strcmp(mdjb, "they") == 0){
        ptreplace(mdi+1, 3, "are ");    return;
      }
      else{
        ptreplace(mdi+1, 3, "is ");    return;
      }
    }
    else if(strncmp(mdi+1, "was", 3) == 0){
      if(strstr(mzj, "") || strcmp(mdj+1, "you") == 0){
        ptreplace(mdi+1, 3, "were");    return;
      }
    }
    else if(strncmp(mdi+1, "have", 4) == 0){
      if((strstr(mzj, "") == 0 && strcmp(mdj+1, "I") != 0
	  && strcmp(mdj+1, "you")  && strcmp(mdj+1, "we") )
      || strcmp(mdj+1, "he") == 0 || strcmp(mdj+1, "she") == 0){
        if(strchr(mzj, 'n') ) return;
        ptreplace(mdi+1, 4, "has");   return;
      }
    }
    else{
    if(Dbg == 'Y') fprintf(dbg, "--- ninsyo --- 1 %s %s \n", mdj+1, mzj);
      if(strcmp(mdjb, "SJ") == 0){ strcpy(mdj, sj);  strcpy(mzj, sz); }
      if(strchr(mzi, 'p')) return;
      if(strstr(mzj, "") || strchr(mzj, 'n') || strchr(mzj, 'x')) return;
      if(strcmp(mdj+1, "I") == 0 || strcmp(mdj+1, "you") == 0) return;
      if(strcmp(mdj+1, "we") == 0 || strcmp(mdj+1, "they") == 0) return;
      if(strstr(mdj+1, "men") || strstr(mdj+1, " and ")) return;
      else if(strstr("he,she,it", mdj+1) || strstr(mzj, "") == 0 || strchr(mzj, 'z') ){
    if(Dbg == 'Y') fprintf(dbg, "--- ninsyo --- 2 %s \n", mdj+1);
        if(strcmp(mdi+1, "often happen") == 0){
	  ptreplace(mdi+1, 12, "often happens");  return;
        }
        if((pt = (char*)strchr(mdi, ' ')) || (pt = (char*)strchr(mdi, '\0'))){
          if(*(pt-1) == 'o' || *(pt-1) == 's' || *(pt-1) == 'x'
          || strncmp(pt-2, "sh", 2) == 0 || strncmp(pt-2, "ch", 2) == 0)
             ptinsert(pt, "es");
          else if(*(pt-1) == 'y' && strchr("aiueo", *(pt-2)) == 0)
             ptreplace(pt-1, 1, "ies");
          else ptinsert(pt, "s");
        }
      }
    }
    return;
    NINS0: // ̏ꍇ
    if(*mdi != '1') goto NINS1;
    strcpy(mdjb, mdj+1);
    if(Dbg == 'Y') fprintf(dbg, "--- ninsyo --- 3  %s mdj = %s \n", mdi+1,mdj+1);
    if(strstr(mzj, "") || strchr(mzj, 'n') || strchr(mzj, 'x') || strstr(mdj, "all") || strstr(mdj, " and ")){
//      if((pt = (char*)strchr(mdi, ' ')) || (pt = (char*)strchr(mdi, '\0'))){
      if((pt = (char*)strchr(mdi, '\0'))){
    if(Dbg == 'Y') fprintf(dbg, "--- ninsyo --- 4  %s \n", mdi+1);
        if(strstr(mzi, "") || strchr(mzi,'x') ) return;
        if(strncmp(mdj+1, "1 ", 2) == 0 || strcmp(mdj+1, "1") == 0) return;
        if(strncmp(mdi+1, "sheep", 5) == 0 || strncmp(mdi+1, "deer", 4) == 0
        || strncmp(mdi+1, "Japanese", 8) == 0 || strncmp(mdi+1, "%", 1) == 0
        || strncmp(mdi+1, "yen", 3) == 0
        || strncmp(mdi+1, "o'clock", 7) == 0) return;
	// mdi+1̍ŏ̒Pplu猟v炻̕`Pɒu^[
	if(irregPlu(mdi+1) == 1) return;
        else if(strncmp(mdi+1, "mouse", 5) == 0) ptreplace(mdi+2, 2, "i");
        else if(strncmp(mdi+1, "foot", 4) == 0) ptreplace(mdi+2, 2, "ee");
        else if(strncmp(mdi+1, "tooth", 5) == 0) ptreplace(mdi+2, 2, "ee");
        else if(strncmp(mdi+1, "child", 5) == 0) ptinsert(mdi+6, "ren");
        else if(strncmp(mdi+1, "ox", 5) == 0) ptinsert(mdi+3, "en");
        else if(strncmp(mdi+1, "radio", 5) == 0
        || strncmp(mdi+1, "piano", 5) == 0
        || strncmp(mdi+1, "boy", 3) == 0 || strncmp(mdi+1, "toy", 3) == 0
        || strncmp(mdi+1, "roof", 4) == 0 || strncmp(mdi+1, "day", 3) == 0
        || strncmp(mdi+1, "birthday", 8) == 0
        || strncmp(mdi+1, "handkerchief", 12) == 0)
          ptinsert(pt, "s");
        else if(*(pt-1) == 'o' || *(pt-1) == 's' || *(pt-1) == 'x'
        || strncmp(pt-2, "sh", 2) == 0 || strncmp(pt-2, "ch", 2) == 0)
          ptinsert(pt, "es");
        else if(*(pt-1) == 'y') ptreplace(pt-1, 1, "ies");
        else ptinsert(pt, "s");
      }
    }
    else{
      if(strncmp(mdi+1, "the", 3) == 0 || strncmp(mdi+1, "someone", 7) == 0
      || strncmp(mdi+1, "all", 3) == 0) return;
      if(strchr(mzi, 'K') || strstr(mdj+1, " and ") ) return;
//  printf("--- ninsho --- mdi = %s mzi = %s \n", mdi, mzi);
      if(strncmp(mdi+1, "a ", 2) == 0 || strncmp(mdi+1, "an ", 3) == 0) return;
      if(strchr("aiueo", *(mdi+1))){
        ptinsert(mdi+1, "an ");
      }
      else{
        ptinsert(mdi+1, "a ");
      }
    }
    NINS1: ;
}

// one̓Iȑ㖼ɒu
oneTan(str, ptone)
char *str, *ptone;
{
    int j;
    char str1[ZokuLen];

    if(*str == 'J'){
      j = *(str+1) - '0';
      strcpy(str, md[j]+1);
      strcpy(str1, mz[j]);
   if(Dbg == 'Y') fprintf(dbg, "--- oneTan --- str = %s ptone = %s \n", str,ptone);
      if(strchr(mz[j], 'K') || strchr(mz[j], 'h')){
        if(strchr(mz[j], 'W')){ ptreplace(ptone, 3, "her");    return; }
        else {
          if(strncmp(ptone, "one's", 5) == 0) {
            ptreplace(ptone, 5, "his");    return;
          }
	  //          ptreplace(ptone, 3, "him");    return; 
        }
      }
    }
   if(Dbg == 'Y') fprintf(dbg, "--- oneTan --- str = %s \n", str);

    if(strncmp(ptone, "oneself", 7) == 0) {
      if(!strcmp(str, "I") || !strcmp(str, "me")){
        ptreplace(ptone, 3, "my");    return;
      }
      if(!strcmp(str, "we") || !strcmp(str, "us")){
        ptreplace(ptone, 7, "ourselves");    return;
      }
      if(!strcmp(str, "you")){
        if(strstr(str1, "P")){ ptreplace(ptone, 3, "your"); return; }
        else{ ptreplace(ptone, 7, "yourselves");    return; }
      }
      if(!strcmp(str, "he") || !strcmp(str, "him")){
        ptreplace(ptone, 3, "him");    return;
      }
      if(!strcmp(str, "she") || !strcmp(str, "her")){
        ptreplace(ptone, 3, "her");    return;
      }
      if(!strcmp(str, "it")){
        ptreplace(ptone, 3, "it");    return;
      }
      if(!strcmp(str, "they") || !strcmp(str, "them")){
        ptreplace(ptone, 7, "themselves");    return;
      }
    }

    if(strncmp(ptone, "one's", 5) == 0) {
      if(!strcmp(str, "I") || !strcmp(str, "me")){
        ptreplace(ptone, 5, "my");    return;
      }
      if(!strcmp(str, "we") || !strcmp(str, "us")){
        ptreplace(ptone, 5, "our");    return;
      }
      if(!strcmp(str, "you")){
        ptreplace(ptone, 5, "your"); return;
      }
      if(!strcmp(str, "he") || !strcmp(str, "him")){
        ptreplace(ptone, 5, "his");    return;
      }
      if(!strcmp(str, "she") || !strcmp(str, "her")){
        ptreplace(ptone, 5, "her");    return;
      }
      if(!strcmp(str, "they") || !strcmp(str, "them")){
        ptreplace(ptone, 5, "their");    return;
      }
    }
}

sinko(md)
char md[];
{
    char *pt;
    if(pt = (char*)strchr(md, ' '));
    else pt = (char*)strchr(md, '\0');
    if(strncmp(pt-4, "also ", 5) == 0){
      if(pt = (char*)strchr(pt+1, ' '));
      else return;
    }
 if(Dbg == 'Y') fprintf(dbg, "--- sinko --- md = %s pt = %s \n", md, pt);
    if(strncmp(md+1, "be", 2) == 0 && pt-3 == md){
      ptinsert(pt, "ing");  return;
    }
    if(strncmp(pt-3, "lie", 3) == 0 || strncmp(pt-3, "die", 3) == 0)
      ptreplace(pt-2, 2, "ying");
    else if(strncmp(pt-5, "begin", 5) == 0) ptinsert(pt, "ning");
    else if(strncmp(pt-4, "stop", 4) == 0 || strncmp(pt-4, "skip", 4) == 0) ptinsert(pt, "ping");
    else if(strncmp(pt-4, "swim", 4) == 0) ptinsert(pt, "ming");
    else if(strncmp(pt-3, "cut", 3) == 0) ptinsert(pt, "ting");
    else if(strncmp(pt-3, "get", 3) == 0) ptinsert(pt, "ting");
    else if(strncmp(pt-3, "put", 3) == 0) ptinsert(pt, "ting");
    else if(strncmp(pt-3, "run", 3) == 0) ptinsert(pt, "ning");
    else if(strncmp(pt-3, "sit", 3) == 0) ptinsert(pt, "ting");
    else if(strncmp(pt-3, "see", 3) == 0) ptinsert(pt, "ing");
    else if(*(pt-1) == 'e') ptreplace(pt-1, 1, "ing");
    else ptinsert(pt, "ing");
}

hikaku(md)
char md[];
{
    char *pt;
    if(pt = (char*)strchr(md, ' '));
    else pt = (char*)strchr(md, '\0');
    if(strncmp(md+1, "beautiful", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "carefully", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "difficult", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "excellent", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "expensive", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "famous", 6) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "important", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "interesting", 11) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "popular", 7) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "quickly", 7) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "slowly", 6) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "useful", 6) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(md+1, "wonderful", 9) == 0){ ptinsert(md+1, "more "); return; }
    if(strncmp(pt-4, "good", 4) == 0 || strncmp(pt-4, "well", 4) == 0)
      ptreplace(pt-4, 4, "better");
    else if(strncmp(pt-4, "many", 4) == 0 || strncmp(pt-4, "much", 4) == 0)
      ptreplace(pt-4, 4, "more");
    else if(strncmp(pt-3, "bad", 3) == 0 || strncmp(pt-3, "ill", 3) == 0)
      ptreplace(pt-3, 3, "worse");
    else if(strncmp(pt-6, "little", 6) == 0) ptreplace(pt-6, 6, "less");
    else if(strncmp(pt-3, "big", 3) == 0) ptinsert(pt, "ger");
    else if(strncmp(pt-3, "hot", 3) == 0) ptinsert(pt, "ter");
    else if(*(pt-1) == 'e') ptinsert(pt, "r");
    else if(*(pt-1) == 'y') ptreplace(pt-1, 1, "ier");
    else ptinsert(pt, "er");    
}

saijyo(md)
char md[];
{
    char *pt;
    if(pt = (char*)strchr(md, ' '));
    else pt = (char*)strchr(md, '\0');
    if( strncmp(md+1, "beautiful", 9) == 0
    || strncmp(md+1, "carefully", 9) == 0
    || strncmp(md+1, "difficult", 9) == 0
    || strncmp(md+1, "excellent", 9) == 0
    || strncmp(md+1, "expensive", 9) == 0
    || strncmp(md+1, "famous", 6) == 0
    || strncmp(md+1, "important", 9) == 0
    || strncmp(md+1, "impressive", 10) == 0
    || strncmp(md+1, "interesting", 11) == 0
    || strncmp(md+1, "popular", 7) == 0
    || strncmp(md+1, "quickly", 7) == 0
    || strncmp(md+1, "slowly", 6) == 0
    || strncmp(md+1, "useful", 6) == 0
    || strncmp(md+1, "wonderful", 9) == 0 ){
      ptinsert(md+1, "the most ");   return;
    }
    if(strncmp(pt-4, "good", 4) == 0 || strncmp(pt-4, "well", 4) == 0)
      ptreplace(pt-4, 4, "best");
    else if(strncmp(pt-4, "many", 4) == 0 || strncmp(pt-4, "much", 4) == 0)
      ptreplace(pt-4, 4, "most");
    else if(strncmp(pt-3, "bad", 3) == 0 || strncmp(pt-3, "ill", 3) == 0)
      ptreplace(pt-3, 3, "worst");
    else if(strncmp(pt-6, "little", 6) == 0) ptreplace(pt-6, 6, "least");
    else if(strncmp(pt-3, "big", 3) == 0) ptinsert(pt, "gest");
    else if(strncmp(pt-3, "hot", 3) == 0) ptinsert(pt, "test");
    else if(*(pt-1) == 'e') ptinsert(pt, "st");
    else if(*(pt-1) == 'y') ptreplace(pt-1, 1, "iest");
    else ptinsert(pt, "est"); 
    if(*md == '5') ptinsert(md+1, "the ");
}
/*
renyo(i)
int  i;
{
    int  j, ln;
    char *gobi, pars[JWordLen], *yk, *qt;
    yk = md[i];
    ln = strlen(yk);
    if(strcmp(yk+ln-2, ") ") == 0 && (qt = (char*)strstr(yk, " ("))){
      strcpy(pars, qt);
      *qt = '\0';
    }
    else *pars = '\0';
    ln = strlen(yk);
    if(ln >= 6){
      gobi = yk + ln -6;
      if(strcmp(gobi, "낤") == 0){
        *(yk + ln -6) = '\0';
        ln -= 6;
        if(ln >= 6) gobi -= 6;
      }
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "");
        goto RTN;
      }
      if(strcmp(gobi, "܂") == 0){
        *gobi = '\0';
        goto RTN;
      }
      if(strcmp(gobi, "Ȃ") == 0){
        strcpy(gobi+4, "");
        goto RTN;
      }
    }
    gobi = yk+ln-4;
    if(strcmp(gobi, "") == 0 ||strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0){
      strcpy(gobi+2, "");	goto RTN;
    }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	goto RTN; }
    if(strcmp(gobi+2, "") == 0){ strcpy(gobi+2, "");	goto RTN; }
    if(strcmp(gobi+2, "") == 0 || strcmp(gobi+2, "") == 0){
      strcpy(gobi+2, "");	goto RTN;
    }
    for(j = 0; j <= 31; j++)
      if(strcmp(gobi, ksm[j]) == 0){ *(gobi+2) = '\0';	goto RTN; }
    gobi += 2;
    for(j = 0; j <= 8; j++)
      if(strcmp(gobi, shs[j]) == 0) strcpy(gobi, rny[j]);
    RTN:
    if(*pars) strcat(yk, pars);
    strcpy(md[i], yk);
}
*/
nega(yk)
char yk[];
{
    int  j, ln;
    char pars[25], daro[7];
    ln = strlen(yk);
    if(strncmp(yk+1, "can", 3) == 0) ptinsert(yk+4, " not");
    else if(strncmp(yk+1, "could", 5) == 0) ptinsert(yk+6, " not");
    else if(strncmp(yk+1, "should", 6) == 0) ptinsert(yk+7, " not");
    else if(strncmp(yk+1, "may", 3) == 0) ptinsert(yk+4, " not");
    else if(strncmp(yk+1, "must", 4) == 0) ptinsert(yk+5, " not");
    else if(strncmp(yk+1, "will", 4) == 0) ptinsert(yk+5, " not");
    else if(strncmp(yk+1, "be ", 3) == 0) ptinsert(yk+4, "not");
    else if(strncmp(yk+1, "be", 2) == 0 && strlen(yk+1) == 2) ptinsert(yk+3, " not");
    else if(strncmp(yk+1, "is ", 3) == 0) ptinsert(yk+3, " not");
    else if(strncmp(yk+1, "was ", 4) == 0) ptinsert(yk+4, " not");
    else if(strncmp(yk+1, "are ", 4) == 0) ptinsert(yk+4, " not");
    else ptinsert(yk+1, "do not ");
/*    if(yk[ln-1] != ')'){
      if(j = paren(yk)){
	strcpy(pars, yk+j-1);	yk[j-1] = '\0';
      }
      else *pars = '\0';
      strcpy(daro, yk+ln-6);
      if(strcmp(daro, "낤") == 0){
	*(yk+ln-6) = '\0';	ln -= 6;
      }
      else daro[0] = '\0';
      nega2(yk);	strcat(yk, daro);	strcat(yk, pars);
    }
    else {
      genkei(yk);	nega2(yk);	past(yk, sw);
    }*/
}
/*
nega2(yk)
char yk[];
{
    char *gobi;
    int  i, ln;
    ln = strlen(yk);	gobi = yk+ln-2;
    if(strcmp(gobi, "") == 0 || strcmp(gobi, "I") == 0){
      strcat(yk, "ł͂Ȃ");	return; 
    }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "ł͂Ȃ");	return; }
    if(ln >= 6){
      gobi = yk+ln-6;
      if(strcmp(gobi, "")== 0){ strcpy(gobi, "Ȃ( 3)");	return; }
      if(strcmp(gobi, "L")== 0){ strcpy(gobi, "( 3)");	return; }
      if(strcmp(gobi, "ł")== 0){ strcpy(gobi, "ł͂Ȃ");	return; }
    }
    gobi = yk+ln-4;
    if(strcmp(gobi, "Ȃ") == 0) return;
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "Ȃ");	return; }
    if(strcmp(gobi, "L") == 0){ strcpy(gobi, "");	return; }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "Ȃ");	return; }
    if(strcmp(gobi, "") == 0){ strcpy(gobi, "Ȃ");	return; }
    if(strcmp(gobi, "܂") == 0){ strcpy(gobi, "܂");	return; }
    for(i = 0; i <= 31; i++)
      if(strcmp(gobi, ksm[i]) == 0){
	strcpy(gobi+2, "Ȃ");	return;
      }
    gobi += 2;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "Ȃ( 3)");	return;
    }
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "Ȃ");	return;
    }
    for(i = 0; i <= 8; i++)
      if(strcmp(gobi, shs[i]) == 0){
	strcpy(gobi, mzn[i]);	strcat(yk, "Ȃ");	return;
      }
    strcat(yk, "̂ł͂Ȃ");
}

gimu(mr)
char mr[];
{
    register int j;
    int  ln;
    char *gobi, pars[JWordLen];
    ln = strlen(mr);
    if(strcmp(mr+ln-2, ") ") == 0){
    }
    else *pars = '\0';
    ln = strlen(mr);	strcpy(pars, "Ȃ΂ȂȂ");
    if(ln >= 4){
      gobi = mr+ln-4;
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "");	goto RTN;
      }
      if(strcmp(gobi, "") == 0){
	strcpy(gobi, "");	goto RTN;
      }
      for(j = 0; j <= 31; j++)
	if(strcmp(gobi, ksm[j]) == 0){
	  strcpy(gobi+2, "");	goto RTN;
	}
    }
    gobi = mr+ln-2;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	goto RTN;
    }
    for(j = 0; j <=8; j++)
      if(strcmp(gobi, shs[j]) == 0){
	strcpy(gobi, mzn[j]);	strcat(gobi+2, "");	goto RTN;
      }
    RTN:
    if(*pars) strcat(mr, pars);
}

rareru(md)
char md[];
{
    register int i;
    char *gobi;
    int  ln;
    ln = strlen(md);
    gobi = md+ln-2;
    if(ln > 5){
      gobi = md+ln-6;
      if(strcmp(gobi, "ł") == 0) return;
      if(strcmp(gobi, "") == 0) return;
      if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	return; }
      if(strcmp(gobi, "") == 0){ strcpy(gobi, "");	return; }
    }
    gobi = md+ln-4;
    if(strcmp(gobi, "") == 0){
      strcpy(gobi, "");	return;
    }
    for(i = 0; i <= 31; i++){
      if(strcmp(gobi, ksm[i]) == 0){
	strcpy(gobi+2, "");	return;
      }
    }
    gobi += 2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi, shs[i]) == 0){
	strcpy(gobi, mzn[i]);	strcat(gobi, "");	return;
      }
    }
}

saser(md)
char md[];
{
    int  i, sw, ln;
    char pars[JWordLen], *pt, *gobi;
    *pars = '\0';
    if(pt = (char*)strstr(md, "i")){
      strcpy(pars, pt);	*pt = '\0';
    }
    if(index(md, '(')) sw = 1;	else sw = 0;
    genkei(md);
    ln = strlen(md);
    if(ln > 3){
      if(strcmp(md, "3") == 0){
	strcpy(md, "3点");	goto RTN;
      }
      gobi = md+ln-4;
      if(strcmp(gobi, "") == 0 || strcmp(gobi, "Ȃ") == 0){
	if(sw == 1) strcpy(gobi, "Ă炤");
	else strcpy(gobi, "");
	goto RTN;
      }
      for(i = 0; i <= 31; i++){
	if(strcmp(gobi,ksm[i]) == 0){
	  if(sw == 1) strcpy(gobi+2, "Ă炤");
	  else strcpy(gobi+2, "");
	  goto RTN;
        }
      }
    }
    gobi = md+ln-2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi, shs[i]) == 0){
        strcpy(gobi, mzn[i]);
        if(sw == 1) strcat(md, "Ă炤");
        else strcat(md, "");
        break;
      }
    }
    RTN:	strcat(md, pars);
}

sei_mei(n1, n2)
char n1[20], n2[20];
{
    unsigned char fst, fam, *pt;
    fst = n1[1];	fam = n2[1];
// printf("--- sei_mei ---  fst = %d  fam = %d \n", fst, fam);
    if((fst < 65 && fst != 32) || (fst > 90 && fst != 131) ||
       (fam < 65 && fam != 32) || (fam > 90 && fam != 131)) swap(n1, n2);
    else{
//      ptinsert(n1+1, " ");
	strcat(n1, " ");
    }
    while(pt = (char*)strstr(n1, "  ")) ptcut(pt, 1);
    while(pt = (char*)strstr(n2, "  ")) ptcut(pt, 1);
}

desu(mr)
char mr[];
{
    int  i, ln, ln1;
    char z[3], k[18], *gobi;
    ln = strlen(mr);
    gobi = mr + ln;
    strcpy(z, gobi-2);
    if(strcmp(z, "A") == 0){ *z = '\0';	goto RTN; }
    if(strcmp(z, "") == 0 || strcmp(z, "B") == 0){
      ptcut(gobi-2, 2);	ln -= 2;
    }
    else *z = '\0';
    gobi = mr+ln;
    for(i = 0; i <= 13; i++){
      ln1 = strlen(bungo[i]);
      if(ln > ln1-1 && strcmp(gobi-ln1, bungo[i]) == 0){
	strcpy(gobi-ln1, kougo[i]);	goto RTN;
      }
    }
    for(i = 0; i <= 31; i++){
      strcpy(k, ksm[i]);
      if(ln > 3 && strcmp(gobi-4, k) == 0){
	strcpy(gobi-2, "܂");	goto RTN;
      }
      strcpy(k+2, "Ȃ");
      if(ln > 5 && strcmp(gobi-6, k) == 0){
        if(strncmp(k, "", 2) == 0) break;
	strcpy(gobi-4, "܂");	goto RTN;
      }
      strcpy(k+4, "( 3)");
      if(ln > 15 && strcmp(gobi-16, k) == 0){
	ptreplace(gobi-14, 14, "܂ł");	goto RTN;
      }

      strcpy(k+2, "( 1)");
      if(ln > 9 && strcmp(gobi-10, k) == 0){
	strcpy(gobi-8, "܂");	goto RTN;
      }
    }
    for(i = 0; i <= 8; i++){
      if(ln > 1 && strcmp(gobi-2, shs[i]) == 0){
	strcpy(gobi-2, rny[i]);	strcat(mr, "܂");	goto RTN;
      }
      strcpy(k, mzn[i]);	strcat(k, "Ȃ");
      if(ln > 5 && strcmp(gobi-6, k) == 0){
	ptreplace(gobi-6, 6, rny[i]);	strcat(mr, "܂");	goto RTN;
      }
      strcpy(k+4, "( 3)");
      if(ln > 15 && strcmp(gobi-16, k) == 0){
	strcpy(gobi-16, rny[i]);	strcat(mr, "܂ł");	goto RTN;
      }
      *k = '\0';
      strink(k, "(", shs[i], " 2)", (char*)0);
      if(ln > 5 && strcmp(gobi-6, k) == 0){
	strcpy(gobi-10, rny[i]);	strcat(mr, "܂");	goto RTN;
      }
    }
    strcat(mr, "ł");
    RTN:	strcat(mr, z);
}

dekiru(char *md)
{
    int i, ln;
    char *gobi;
    ln = strlen(md);
    if(ln == 8){
      gobi = md + ln -4;
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "ł");
        return;
      }
    }
    if(ln > 5){
      gobi = md + ln -6;
      if(strcmp(gobi, "") == 0){
        strcpy(gobi, "ł");
        return;
      }
    }
    if(ln > 3){
      gobi = md + ln - 4;
      for(i = 0; i <= 31; i++){
        if(strcmp(gobi, ksm[i]) == 0){
          strcpy(gobi+2, "");
          return;
        }
      }
    }
    gobi = md + ln - 2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi, shs[i]) == 0){
        strcpy(gobi, mei[i]);
        strcat(gobi, "");
        break;
      }
    }
  }

kanyuugobi(md)
char md[];
{
    int  i, ln;
    char *gobi;
    ln = strlen(md);
    if(ln > 3){
      gobi = md+ln-4;
      if(strcmp(gobi,"") == 0){ strcpy(gobi, "悤");	return; }
      for(i = 0; i <= 31; i++){
        if(strcmp(gobi, ksm[i]) == 0){
          strcpy(gobi+2, "悤");
          return;
        }
      }
    }
    gobi = md+ln-2;
    for(i = 0; i <= 8; i++){
      if(strcmp(gobi,shs[i]) == 0){
	strcpy(gobi, kanyu[i]);	return;
      }
    }
}
*/
minus(str, c, i)
int  i;
char c, *str;
{
    char *pt, *pt1, *pt2;
    if(c == 'T')
      while(pt = (char*)strstr(cz[wc+i], str))
	ptcut(pt, 1);
    else {
      if(strcmp(str, "ij") == 0)
	while((pt1 = (char*)strstr(md[i], "i")) &&
	 (pt2 = (char*)strstr(pt1+2,"j"))){
	  pt = pt2+1;
	  do{ *(pt1++) = *(++pt);
	  } while(*pt);
	  return;
	}
      if(strcmp(str, "()") == 0)
	while((pt1 = (char*)index(md[i], '(')) &&
	 (pt2 = (char*)index(pt1+1,')'))){
	  ptptcut(pt1, pt2);
	}
      else
	while(pt = (char*) strstr(md[i], str)) ptcut(pt, strlen(str));
    }
}

add(c, i, str)
int  i;
char c, str[];
{
    char st1[50], *pt;
    strcpy(st1, str+1);
    if(strcmp(st1, "OZ$") == 0){
      if(pt = (char*)rindex(oz, 'y')) strcpy(st1, pt);
      else{
	if(pt = (char*)rindex(sz, 'y')) strcpy(st1, pt);
      }
      while(pt = (char *)index(st1, '?')) ptcut(pt, 1);
      strcat(cz[wc+i], st1);	return;
    }
    if(strcmp(st1, "VR$") == 0) strcpy(st1, vr);
    if(c == 'J'){
      if(*st1 == '@') ptcut(st1, 1);
      else strcat(md[i], " ");
      strcat(md[i], st1);
    }
    else strcat(cz[wc+i], st1);
}
/*
kazu1(i, kaz)
int  i;
char kaz[30];
{
    char bz[30], *pt, *pt2;
    strcpy(bz, cz[wc+i]);
    if(pt = (char*)index(bz, '%')) strcpy(pt, "");
    if(pt = (char*)rindex(bz, 'y')){
      strcpy(kaz, pt+1);
      if((pt2 = (char*)index(kaz, 'b')) && (pt = (char*)index(pt2, 'y')))
	ptptcut(kaz, pt);
    }
    else {
      if(index(bz, 'h')){
	strcpy(bz, "l");	return;
      }
      if(index(bz, 't')){
	kaz[0] = '\0';	return;
      }
    }
}
*/
kazu2(i, kaz)
int  i;
char kaz[30];
{
    char bj[100];
    strcpy(bj, md[i]);
    if(strcmp(bj+1, "1") == 0) strcat(bj, "st");
    else if(strcmp(bj+1, "2") == 0) strcat(bj, "nd");
    else if(strcmp(bj+1, "3") == 0) strcat(bj, "rd");
    else strcat(bj, "th");
    strcpy(md[i],bj);
}

kazu3(i, kaz)
int  i;
char kaz[30];
{
    char bj[100];
    strcpy(bj, md[i]);
    if(strncmp(bj+1, "Japanese", 8) == 0) return;
    strcat(bj, "s");
    strcpy(md[i],bj);
}

irregPlu(wd) // wd̒P
char *wd;
{
  int n;
  char buf[400], *pt, *pt1, *pt2;
  if(strlen(wd) == 0) return(0);
  strcpy(buf, wd);
  if(pt1 = strchr(buf, ' ')) *pt1 = '\0';
  n = strlen(buf);
  pt = plu;
 IP0:
  pt1 = strstr(pt, buf);
  if(pt1 == 0) return(0);
  //  if(*(pt1-1) != ';' && pt1 != plu) return(0);
  if(*(pt1-1) != ';' && pt1 != plu){ pt++; goto IP0; }
  pt2 = strchr(pt1, ';');
  ptcpy(buf, pt1+n+1, pt2);
  //  if(Dbg == 'Y') fprintf(dbg, "--- irregPlu --- buf = %s \n", buf);
  ptreplace(wd, n, buf);
  return(1);
}
/*
kakudel(str)
char str[];
{
    int  i;
    char jos[3];
    if(str[1] == '#'){
      ptcpy(jos, str+3, str+5);	i = 3;
    }
    else {
      ptcpy(jos, str+1, str+3);	i = 1;
    }
    if(strcmp(jos, "")==0 || strcmp(jos, "")==0 || strcmp(jos, "")==0 ||
      strcmp(jos, "")==0 && instr(str, "ƂĂ")!=2) ptcut(str+i, 2);
}
*/
kakumv(st1, st2)
char st1[], st2[];
{
    char jos[10], *pt;
    if((pt = (char*)strstr(st1+1, "not ")) ){
      pt = pt+3;
      ptcpy(jos, st1+1, pt+1);
      if(strcmp(jos, "do not ") == 0){
        ptcut(st1+1, strlen(jos));	ptinsert(st2+1, jos);
      }
      return;
    }
    pt = (char*)strchr(st1+1, ' ');
    ptcpy(jos, st1+1, pt+1);
    if(strcmp(jos, "is ") == 0 || strcmp(jos, "are ") == 0
    || strcmp(jos, "was ") == 0 || strcmp(jos, "were ") == 0
    || strcmp(jos, "can ") == 0 || strcmp(jos, "may ") == 0
    || strcmp(jos, "will ") == 0 || strcmp(jos, "shall ") == 0
    || strcmp(jos, "must ") == 0 || strcmp(jos, "did ") == 0
    || strcmp(jos, "do ") == 0 || strcmp(jos, "does ") == 0
    || strcmp(jos, "have ") == 0 || strcmp(jos, "has ") == 0){
      ptcut(st1+1, strlen(jos));	ptinsert(st2+1, jos);
    }
    return;
}

